import os
import time
import typer
from typing import List, Optional, Tuple
from roseApp.core.parser import create_parser, ParserType
from roseApp.core.util import get_logger, TimeUtil
# from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

import logging

# Initialize logger
logger = get_logger("RoseCLI - filter")
# console = Console()

app = typer.Typer()

@app.command()
def filter(
    input_bag: str = typer.Argument(..., help="Input bag file path"),
    output_bag: str = typer.Argument(..., help="Output bag file path"),
    whitelist: Optional[str] = typer.Option(None, "--whitelist", "-w", help="Path to topic whitelist file"),
    topics: Optional[List[str]] = typer.Option(None, "--topics", "-tp", help="Topics to include (can be specified multiple times). Alternative to whitelist file."),
    dry_run: bool = typer.Option(False, "--dry-run", help="Show what would be done without actually doing it")
):
    """Filter ROS bag by topic whitelist and/or time range.
    
    Examples:
    
        rose filter input.bag output.bag -w whitelist.txt
        rose filter input.bag output.bag -t "23/01/01 00:00:00,23/01/01 00:10:00"
        rose filter input.bag output.bag --topics /topic1 --topics /topic2
    """
    try:
        parser = create_parser(ParserType.PYTHON)
        
        # Check if input file exists
        if not os.path.exists(input_bag):
            typer.echo(f"Error: Input file '{input_bag}' does not exist", err=True)
            raise typer.Exit(code=1)
            
        # Check if whitelist file exists
        if whitelist and not os.path.exists(whitelist):
            typer.echo(f"Error: Whitelist file '{whitelist}' does not exist", err=True)
            raise typer.Exit(code=1)
        
        # Get all topics from input bag
        all_topics, connections, _ = parser.load_bag(input_bag)
        

        # Get topics from whitelist file or command line arguments
        whitelist_topics = set()
        if whitelist:
            whitelist_topics.update(parser.load_whitelist(whitelist))
        if topics:
            whitelist_topics.update(topics)
            
        if not whitelist_topics:
            typer.echo("Error: No topics specified. Use --whitelist or --topics", err=True)
            raise typer.Exit(code=1)
            
        # Show what will be done in dry run mode
        if dry_run:
            typer.secho("DRY RUN - No changes will be made", fg=typer.colors.YELLOW, bold=True)
            typer.echo(f"Would filter {typer.style(input_bag, fg=typer.colors.GREEN)} to {typer.style(output_bag, fg=typer.colors.BLUE)}")
            
            # Show all topics with selection status
            typer.echo("\nTopic Selection:")
            typer.echo("─" * 80)
            for topic in sorted(all_topics):
                is_selected = topic in whitelist_topics
                status_icon = typer.style('✓', fg=typer.colors.GREEN) if is_selected else typer.style('○', fg=typer.colors.YELLOW)
                topic_style = typer.colors.GREEN if is_selected else typer.colors.WHITE
                msg_type_style = typer.colors.CYAN if is_selected else typer.colors.WHITE
                topic_str = f"{topic:<40}"
                typer.echo(f"  {status_icon} {typer.style(topic_str, fg=topic_style)} "
                          f"{typer.style(connections[topic], fg=msg_type_style)}")
            
            return
        
        # Print filter information
        typer.secho("\nStarting bag filter:", bold=True)
        typer.echo(f"Input:  {typer.style(input_bag, fg=typer.colors.GREEN)}")
        typer.echo(f"Output: {typer.style(output_bag, fg=typer.colors.BLUE)}")
        
        # Show all topics with selection status
        typer.echo("\nTopic Selection:")
        typer.echo("─" * 80)
        selected_count = 0
        for topic in sorted(all_topics):
            is_selected = topic in whitelist_topics
            if is_selected:
                selected_count += 1
            status_icon = typer.style('✓', fg=typer.colors.GREEN) if is_selected else typer.style('○', fg=typer.colors.YELLOW)
            topic_style = typer.colors.GREEN if is_selected else typer.colors.WHITE
            msg_type_style = typer.colors.CYAN if is_selected else typer.colors.WHITE
            topic_str = f"{topic:<40}"
            typer.echo(f"  {status_icon} {typer.style(topic_str, fg=topic_style)} "
                      f"{typer.style(connections[topic], fg=msg_type_style)}")
        
        # Show selection summary
        typer.echo("─" * 80)
        typer.echo(f"Selected: {typer.style(str(selected_count), fg=typer.colors.GREEN)} of "
                  f"{typer.style(str(len(all_topics)), fg=typer.colors.WHITE)} topics")
        

        # Run the filter with progress bar
        typer.echo("\nProcessing:")
        start_time = time.time()
        
        # Use Rich's progress bar instead of Click's progress bar
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=False,
        ) as progress:
            task = progress.add_task("Filtering bag file...", total=100)
            result = parser.filter_bag(
                input_bag, 
                output_bag, 
                list(whitelist_topics)
            )
            progress.update(task, completed=100)
        
        # Show filtering results
        end_time = time.time()
        elapsed = end_time - start_time
        input_size = os.path.getsize(input_bag)
        output_size = os.path.getsize(output_bag)
        size_reduction = (1 - output_size/input_size) * 100
        
        typer.secho("\nFilter Results:", fg=typer.colors.GREEN, bold=True)
        typer.echo("─" * 80)
        typer.echo(f"Time taken: {int(elapsed//60)}m {elapsed%60:.2f}s")
        typer.echo(f"Input size:  {typer.style(f'{input_size/1024/1024:.2f} MB', fg=typer.colors.YELLOW)}")
        typer.echo(f"Output size: {typer.style(f'{output_size/1024/1024:.2f} MB', fg=typer.colors.YELLOW)}")
        typer.echo(f"Reduction:   {typer.style(f'{size_reduction:.1f}%', fg=typer.colors.GREEN)}")
        typer.echo(result)
        
    except Exception as e:
        logger.error(f"Error during filtering: {str(e)}", exc_info=True)
        typer.echo(f"Error: {str(e)}", err=True)
        raise typer.Exit(code=1)

def main():
    """Entry point for the CLI tool"""
    app()

if __name__ == "__main__":
    main() 


import os
import time
from typing import Optional, List
from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
import typer
from InquirerPy.validator import PathValidator

from ..core.parser import create_parser, ParserType
from ..core.util import get_logger
from .theme import style, GREEN, YELLOW, BLUE, PURPLE, ORANGE  # Import colors and style
from .util import (build_banner, 
                   collect_bag_files, 
                   print_usage_instructions, 
                   print_bag_info, 
                   print_filter_stats,
                   print_batch_filter_summary)
logger = get_logger("RoseCLI-Tool")

app = typer.Typer(help="ROS Bag Filter Tool")


class CliTool:
    def __init__(self):
        self.console = Console()
        self.parser = create_parser(ParserType.PYTHON)
        self.topics = None
        self.connections = None
        self.time_range = None
 
    def ask_for_bag(self, message: str = "Enter bag file path:") -> Optional[str]:
        """Ask user to input a bag file path"""
        while True:
            input_bag = inquirer.filepath(
                message=message,
                validate=PathValidator(is_file=True, message="File does not exist"),
                filter=lambda x: x if x.endswith('.bag') else None,
                invalid_message="File must be a .bag file",
                style=style
            ).execute()
            
            if input_bag is None:  # User cancelled
                return None
                
            return input_bag
    
    def show_loading(self, message: str):
        """Show a loading spinner with message"""
        return Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
        )
    
    def run_cli(self):
        """Run the CLI tool with improved menu logic"""
        try:
            # Show banner
            self.console.print(build_banner())
            
            while True:
                # Show main menu
                action = inquirer.select(
                    message="Select action:",
                    choices=[
                        Choice(value="filter", name="1. Bag Editor - View and filter bag files"),
                        Choice(value="whitelist", name="2. Whitelist - Manage topic whitelists"),
                        Choice(value="exit", name="3. Exit")
                    ],
                    style=style
                ).execute()
                
                if action == "exit":
                    break
                elif action == "filter":
                    self._run_interactive_filter()
                elif action == "whitelist":
                    self._run_whitelist_manager()
                
        except KeyboardInterrupt:
            self.console.print("\nOperation cancelled by user", style=YELLOW)
        except Exception as e:
            logger.error(f"Error: {str(e)}", exc_info=True)
            self.console.print(f"\nError: {str(e)}", style="red")


    def _run_interactive_filter(self):
        """Run quick filter workflow"""
        while True:
            # Ask for input bag file or directory
            input_path = inquirer.filepath(
                message="Load Bag file(s):\n • Please specify the bag file or a directory to search \n • Leave blank to return to main menu\nFilename/Directory:",
                validate=lambda x: os.path.exists(x) or "Path does not exist",
                style=style
            ).execute()
            
            if not input_path:
                return  # Return to main menu
                
            # Check if input is a file or directory
            if os.path.isfile(input_path):
                # Single bag file processing
                if not input_path.endswith('.bag'):
                    self.console.print("File must be a .bag file", style="red")
                    continue
                    
                # Load bag info
                with self.show_loading("Loading bag file...") as progress:
                    progress.add_task(description="Loading...")
                    self.topics, self.connections, self.time_range = self.parser.load_bag(input_path)
                
                # Create a loop for bag operations
                while True:
                    # Ask user what to do next
                    next_action = inquirer.select(
                        message="What would you like to do?",
                        choices=[
                            Choice(value="info", name="1. Show bag information"),
                            Choice(value="filter", name="2. Filter bag file"),
                            Choice(value="back", name="3. Back to file selection")
                        ],
                        style=style
                    ).execute()
                    
                    if next_action == "back":
                        break  # Go back to input selection
                    elif next_action == "info":
                        print_bag_info(self.console, input_path, self.topics, self.connections, self.time_range)
                        continue  # Stay in the current menu
                    elif next_action == "filter":
                        # Get output bag
                        output_bag = inquirer.filepath(
                            message="Enter output bag file path:",
                            default=os.path.splitext(input_path)[0] + "_filtered.bag",
                            validate=lambda x: x.endswith('.bag') or "File must be a .bag file",
                            style=style
                        ).execute()
                        
                        if not output_bag:
                            continue  # Stay in the current menu
                            
                        # Get filter method
                        filter_method = inquirer.select(
                            message="Select filter method:",
                            choices=[
                                Choice(value="whitelist", name="1. Use whitelist"),
                                Choice(value="manual", name="2. Select topics manually"),
                                Choice(value="back", name="3. Back")
                            ],
                            style=style
                        ).execute()
                        
                        if not filter_method or filter_method == "back":
                            continue  # Stay in the current menu
                            
                        # Process single file
                        self._process_single_bag(input_path, output_bag, filter_method)
                        # Continue in the same menu after processing
                
            else:  # Directory processing
                # Find and select bag files
                bag_files = collect_bag_files(input_path)
                if not bag_files:
                    self.console.print("No bag files found in directory", style="red")
                    continue  # Go back to input selection
                    
                # Create file selection choices
                file_choices = [
                    Choice(
                        value=f,
                        name=f"{os.path.relpath(f, input_path)} ({os.path.getsize(f)/1024/1024:.1f} MB)"
                    ) for f in bag_files
                ]
                
                def bag_list_transformer(result):
                    return f"{len(result)} files selected\n" + '\n'.join([f"• {os.path.basename(bag)}" for bag in result])
                
                print_usage_instructions(self.console)

                selected_files = inquirer.checkbox(
                    message="Select bag files to process:",
                    choices=file_choices,
                    instruction="",
                    validate=lambda result: len(result) > 0,
                    invalid_message="Please select at least one file",
                    transformer=bag_list_transformer,
                    style=style
                ).execute()
                
                if not selected_files:
                    continue  # Go back to input selection
                    
                # Get filter method
                filter_method = inquirer.select(
                    message="Select filter method:",
                    choices=[
                        Choice(value="whitelist", name="1. Use whitelist"),
                        Choice(value="manual", name="2. Select topics manually"),
                        Choice(value="back", name="3. Back")
                    ],
                    style=style
                ).execute()
                
                if not filter_method or filter_method == "back":
                    continue  # Go back to input selection
                    
                

                # Load first bag file to get topics for selection
                whitelist = None
                if filter_method == "whitelist":
                    # Get whitelist file
                    whitelist_dir = "whitelists"
                    if not os.path.exists(whitelist_dir):
                        self.console.print("No whitelists found", style=YELLOW)
                        continue  # Go back to input selection
                        
                    whitelists = [f for f in os.listdir(whitelist_dir) if f.endswith('.txt')]
                    if not whitelists:
                        self.console.print("No whitelists found", style=YELLOW)
                        continue  # Go back to input selection
                        
                    # Select whitelist to use
                    selected = inquirer.select(
                        message="Select whitelist to use:",
                        choices=whitelists,
                        style=style
                    ).execute()
                    
                    if not selected:
                        continue  # Go back to input selection
                        
                    # Load selected whitelist
                    whitelist_path = os.path.join(whitelist_dir, selected)
                    whitelist = self.parser.load_whitelist(whitelist_path)
                    if not whitelist:
                        continue  # Go back to input selection
                        
                elif filter_method == "manual":
                    # Load all bag files to get the union of topics
                    all_topics = set()
                    all_connections = {}
                    
                    with Progress(
                        SpinnerColumn(),
                        TextColumn("[progress.description]{task.description}"),
                        transient=True,
                    ) as progress:
                        task = progress.add_task("Loading bag files for topic selection...", total=len(selected_files))
                        
                        for i, bag_file in enumerate(selected_files):
                            progress.update(task, description=f"Loading {i+1}/{len(selected_files)}: {os.path.basename(bag_file)}")
                            try:
                                topics, connections, _ = self.parser.load_bag(bag_file)
                                all_topics.update(topics)
                                all_connections.update(connections)
                                progress.advance(task)
                            except Exception as e:
                                self.console.print(f"Error loading {bag_file}: {str(e)}", style="red")
                                # Continue with other files
                    
                    if not all_topics:
                        self.console.print("No topics found in selected bag files", style="red")
                        continue  # Go back to input selection
                    
                    self.console.print(f"Found {len(all_topics)} unique topics across {len(selected_files)} bag files", style=GREEN)
                    whitelist = self._select_topics(list(all_topics), all_connections)
                    if not whitelist:
                        continue  # Go back to input selection
                
                # Create progress display for all files
                with Progress(
                    SpinnerColumn(),
                    TextColumn("[progress.description]{task.description}"),
                    transient=False,
                ) as progress:
                    # Create tasks for all files
                    tasks = {}
                    for bag_file in selected_files:
                        rel_path = os.path.relpath(bag_file, input_path)
                        task = progress.add_task(
                            f"Waiting: {rel_path}",
                            total=100,
                            style="dim"
                        )
                        tasks[bag_file] = task
                    
                    # Process each file
                    for bag_file in selected_files:
                        rel_path = os.path.relpath(bag_file, input_path)
                        task = tasks[bag_file]
                        
                        try:
                            # Update task to show it's being processed
                            progress.update(task, description=f"Processing: {rel_path}", style="yellow")
                            
                            # Create output path
                            output_bag = os.path.splitext(bag_file)[0] + "_filtered.bag"
                            
                            # Process file with the selected whitelist
                            self._process_single_bag(
                                bag_file,
                                output_bag,
                                filter_method,
                                whitelist=whitelist,  # Pass the pre-selected whitelist
                                progress_context=progress,
                                task_id=task
                            )
                            
                            # Update task to show success with green color
                            progress.update(task, description=f"[green]✓ {rel_path}[/green]")
                            
                        except Exception as e:
                            # Update task to show failure with red color
                            progress.update(task, description=f"[red]✗ {rel_path}: {str(e)}[/red]")
                            logger.error(f"Error processing {bag_file}: {str(e)}", exc_info=True)
                        
                        # Update progress
                        progress.update(task, completed=100)
                        
                # Show final summary with color-coded results
                print_batch_filter_summary(self.console, tasks, progress)
                
                # Ask if user wants to continue or go back to main menu
                continue_action = inquirer.select(
                    message="What would you like to do next?",
                    choices=[
                        Choice(value="continue", name="1. Process more files"),
                        Choice(value="main", name="2. Return to main menu")
                    ],
                    style=style
                ).execute()
                
                if continue_action == "main":
                    return  # Return to main menu

    def _process_single_bag(self, input_bag: str, output_bag: str, filter_method: str, 
                          whitelist: Optional[List[str]] = None,
                          progress_context: Optional[Progress] = None, task_id: Optional[int] = None):
        """Process a single bag file"""
        # Load bag info
        if progress_context:
            # In batch mode, use the provided progress context
            progress_context.update(task_id, description=f"Loading: {os.path.basename(input_bag)}")
            self.topics, self.connections, self.time_range = self.parser.load_bag(input_bag)
        else:
            # In single file mode, use independent loading animation
            with self.show_loading("Loading bag file...") as progress:
                progress.add_task(description="Loading...")
                self.topics, self.connections, self.time_range = self.parser.load_bag(input_bag)
        
        # Get filter parameters based on method if not provided
        if whitelist is None:
            if filter_method == "whitelist":
                # Get whitelist file
                whitelist_dir = "whitelists"
                if not os.path.exists(whitelist_dir):
                    self.console.print("No whitelists found", style="yellow")
                    return
                    
                whitelists = [f for f in os.listdir(whitelist_dir) if f.endswith('.txt')]
                if not whitelists:
                    self.console.print("No whitelists found", style="yellow")
                    return
                    
                # Select whitelist to use
                selected = inquirer.select(
                    message="Select whitelist to use:",
                    choices=whitelists,
                    style=style
                ).execute()
                
                if not selected:
                    return
                    
                # Load selected whitelist
                whitelist_path = os.path.join(whitelist_dir, selected)
                whitelist = self.parser.load_whitelist(whitelist_path)
                if not whitelist:
                    return
                    
            elif filter_method == "manual":
                whitelist = self._select_topics(self.topics, self.connections)
                if not whitelist:
                    return
                
        # Filter bag
        if progress_context:
            # In batch mode, use the provided progress context
            progress_context.update(task_id, description=f"Filtering: {os.path.basename(input_bag)}")
            self.parser.filter_bag(input_bag, output_bag, whitelist)
        else:
            # In single file mode, use independent loading animation
            with self.show_loading("Filtering bag file...") as progress:
                progress.add_task(description="Processing...")
                self.parser.filter_bag(input_bag, output_bag, whitelist)
        
        # Show results
        if not progress_context:  # Only show stats for single file processing
            print_filter_stats(self.console, input_bag, output_bag)
            
        
    def _run_whitelist_manager(self):
        """Run whitelist management workflow"""
        while True:
            action = inquirer.select(
                message="Whitelist Management:",
                choices=[
                    Choice(value="create", name="1. Create new whitelist"),
                    Choice(value="view", name="2. View whitelist"),
                    Choice(value="delete", name="3. Delete whitelist"),
                    Choice(value="back", name="4. Back")
                ],
                style=style
            ).execute()
            
            if action == "back":
                return
            elif action == "create":
                self._create_whitelist_workflow()
            elif action == "view":
                self._browse_whitelists()
            elif action == "delete":
                self._delete_whitelist()
    
    def _create_whitelist_workflow(self):
        """Create whitelist workflow"""
        # Get bag file
        input_bag = self.ask_for_bag("Enter bag file path to create whitelist from:")
        if not input_bag:
            return
            
        # Load bag file
        with self.show_loading("Loading bag file...") as progress:
            progress.add_task(description="Loading...")
            topics, connections, _ = self.parser.load_bag(input_bag)
        
        # Select topics
        selected_topics = self._select_topics(topics, connections)
        if not selected_topics:
            return
            
        # Save whitelist
        timestamp = time.strftime("%Y%m%d_%H%M%S")
        default_path = f"whitelists/whitelist_{timestamp}.txt"
        
        use_default = inquirer.confirm(
            message=f"Use default path? ({default_path})",
            default=True,
            style=style
        ).execute()
        
        if use_default:
            output = default_path
        else:
            output = inquirer.filepath(
                message="Enter save path:",
                default="whitelists/my_whitelist.txt",
                validate=lambda x: x.endswith('.txt') or "File must be a .txt file",
                style=style
            ).execute()
            
            if not output:
                return
        
        # Save whitelist
        os.makedirs(os.path.dirname(output) if os.path.dirname(output) else '.', exist_ok=True)
        with open(output, 'w') as f:
            f.write("# Generated by rose cli-tool\n")
            f.write(f"# Date: {time.strftime('%Y-%m-%d %H:%M:%S')}\n")
            f.write("\n")
            for topic in sorted(selected_topics):
                f.write(f"{topic}\n")
        
        self.console.print(f"\nSaved whitelist to: {output}", style="green")
        
        # Ask what to do next
        next_action = inquirer.select(
            message="What would you like to do next?",
            choices=[
                Choice(value="continue", name="1. Create another whitelist"),
                Choice(value="back", name="2. Back")
            ],
            style=style
        ).execute()
        
        if next_action == "continue":
            self._create_whitelist_workflow()
    
    def _browse_whitelists(self):
        """Browse and view whitelist files"""
        # Get all whitelist files
        whitelist_dir = "whitelists"
        if not os.path.exists(whitelist_dir):
            self.console.print("No whitelists found", style="yellow")
            return
            
        whitelists = [f for f in os.listdir(whitelist_dir) if f.endswith('.txt')]
        if not whitelists:
            self.console.print("No whitelists found", style="yellow")
            return
            
        # Select whitelist to view
        selected = inquirer.select(
            message="Select whitelist to view:",
            choices=whitelists,
            style=style
        ).execute()
        
        if not selected:
            return
            
        # Show whitelist contents
        path = os.path.join(whitelist_dir, selected)
        with open(path) as f:
            content = f.read()
            
        self.console.print(f"\nWhitelist: {selected}", style="bold green")
        self.console.print("─" * 80)
        self.console.print(content)
    
    def _select_topics(self, topics: List[str], connections: dict) -> Optional[List[str]]:
        """Select topics manually"""
        topic_choices = sorted(topics)
        
        # Display usage instructions
        print_usage_instructions(self.console, is_fuzzy=True)
        
        selected_topics = inquirer.fuzzy(
            message="Select topics to include:",
            choices=topic_choices,
            multiselect=True,
            validate=lambda result: len(result) > 0,
            invalid_message="Please select at least one topic",
            transformer=lambda result: f"{len(result)} topic{'s' if len(result) > 1 else ''} selected",
            max_height="70%",
            instruction="",
            marker="● ",
            border=True,
            cycle=True,
            style=style
        ).execute()
        
        return selected_topics
    

        
    def _delete_whitelist(self):
        """Delete a whitelist file"""
        whitelist_dir = "whitelists"
        if not os.path.exists(whitelist_dir):
            self.console.print("No whitelists found", style=YELLOW)
            return
            
        whitelists = [f for f in os.listdir(whitelist_dir) if f.endswith('.txt')]
        if not whitelists:
            self.console.print("No whitelists found", style=YELLOW)
            return
            
        # Select whitelist to delete
        selected = inquirer.select(
            message="Select whitelist to delete:",
            choices=whitelists,
            style=style
        ).execute()
        
        if not selected:
            return
            
        # Confirm deletion
        if not inquirer.confirm(
            message=f"Are you sure you want to delete '{selected}'?",
            default=False,
            style=style
        ).execute():
            return
            
        # Delete the file
        path = os.path.join(whitelist_dir, selected)
        try:
            os.remove(path)
            self.console.print(f"\nDeleted whitelist: {selected}", style=GREEN)
        except Exception as e:
            self.console.print(f"\nError deleting whitelist: {str(e)}", style="red")

# Typer commands
@app.command()
def cli():
    """Interactive CLI mode with menu interface"""
    tool = CliTool()
    tool.run_cli()


def main():
    """Entry point for the CLI tool"""
    app()

if __name__ == "__main__":
    main() 


from rich.panel import Panel
from rich.text import Text
from .theme import style, GREEN, YELLOW, BLUE, PURPLE, ORANGE  # Import colors and style
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
import os
from typing import List
ROSE_BANNER = """
██████╗  ██████╗ ███████╗███████╗
██╔══██╗██╔═══██╗██╔════╝██╔════╝
██████╔╝██║   ██║███████╗█████╗  
██╔══██╗██║   ██║╚════██║██╔══╝  
██║  ██║╚██████╔╝███████║███████╗
╚═╝  ╚═╝ ╚═════╝ ╚══════╝╚══════╝
"""

def build_banner():
    """Display the ROSE banner"""
    # Create title with link
    title = Text()
    title.append("ROS Bag Filter Tool") 
    subtitle = Text()
    subtitle.append("Github", style=f"{YELLOW} link https://github.com/hanxiaomax/rose")
    subtitle.append(" • ", style="dim")
    subtitle.append("Author", style=f"{YELLOW} link https://github.com/hanxiaomax")

    # Create banner content
    content = Text()
    content.append(ROSE_BANNER, style="")
    content.append("Yet another cross-platform and ROS Environment independent editor/filter tool for ROS bag files", style=f"dim {GREEN}")
    
    # Create panel with all elements
    panel = Panel(
        content,
        title=title,
        subtitle=subtitle,  
        border_style=YELLOW,  
        highlight=True
    )
    
    # Print the panel
    # self.console.print(panel)
    return panel
  
def print_usage_instructions(console:Console, is_fuzzy:bool = False):
    console.print("\nUsage Instructions:",style="bold magenta")
    if is_fuzzy:
        console.print("•  [magenta]Type to search[/magenta]")
    else:
        console.print("•  [magenta]Space[/magenta] to select/unselect") 
    console.print("•  [magenta]↑/↓[/magenta] to navigate options")
    console.print("•  [magenta]Tab[/magenta] to select and move to next item")
    console.print("•  [magenta]Shift+Tab[/magenta] to select and move to previous item")
    console.print("•  [magenta]Ctrl+A[/magenta] to select all")
    console.print("•  [magenta]Enter[/magenta] to confirm selection\n")


def collect_bag_files(directory: str) -> List[str]:
    """Recursively find all bag files in the given directory"""
    bag_files = []
    for root, _, files in os.walk(directory):
        for file in files:
            if file.endswith('.bag'):
                bag_files.append(os.path.join(root, file))
    return sorted(bag_files)

def print_bag_info(console:Console, bag_path: str, topics: List[str], connections: dict, time_range: tuple):
    """Show bag file information using rich panels"""
    # Calculate file info
    file_size = os.path.getsize(bag_path)
    file_size_mb = file_size / (1024 * 1024)
    
    # Create topics panel
    bag_info = Text()
    bag_info.append(f"File: {os.path.basename(bag_path)}\n", style=f"bold {GREEN}")
    bag_info.append(f"Size: {file_size_mb:.2f} MB ({file_size:,} bytes)\n")
    bag_info.append(f"Path: {os.path.abspath(bag_path)}\n")
    bag_info.append(f"Topics({len(topics)} in total):\n",style="bold")
    
    for topic in sorted(topics):
        bag_info.append(f"• {topic:<40}", style=f"{PURPLE}")
        bag_info.append(f"{connections[topic]}\n", style="dim")
    panel = Panel(bag_info,
                        title=f"Bag Information",
                        border_style=BLUE,
                        padding=(0, 1))
    

    console.print(panel)
    
def print_filter_stats(console:Console, input_bag: str, output_bag: str):
    """Show filtering statistics"""
    input_size = os.path.getsize(input_bag)
    output_size = os.path.getsize(output_bag)
    input_size_mb:float = input_size / (1024 * 1024)
    output_size_mb:float = output_size / (1024 * 1024)
    reduction_ratio = (1 - output_size / input_size) * 100
    
    stats = (
        f"Filter Statistics:\n"
        f"• Size: {input_size_mb:.2f} MB -> {output_size_mb:.2f} MB\n"
        f"• Reduction: {reduction_ratio:.1f}%\n"
    )
    console.print(Panel(stats, style=GREEN, title="Filter Results"))

def print_batch_filter_summary(console:Console, tasks: dict, progress: Progress):
    """Show filtering results for batch processing"""
    success_count = sum(1 for task in tasks.values() if "✓" in progress.tasks[task].description)
    fail_count = sum(1 for task in tasks.values() if "✗" in progress.tasks[task].description)
    
    summary = (
        f"Processing Complete!\n"
        f"• Successfully processed: {success_count} files\n"
        f"• Failed: {fail_count} files"
    )
    
    if fail_count == 0:
        console.print(Panel(summary,style="green", title="[bold]Results[/bold]"))
    else:
        console.print(Panel(summary,style="red", title="[bold]Results[/bold]"))
